#include "splash-tree-view.h"

#include <glib-object.h>
#include <gtk/gtk.h>
#include <gconf/gconf-client.h>
#include <libgnomevfs/gnome-vfs.h>

G_DEFINE_TYPE (SplashTreeView,
	       splash_tree_view,
	       GTK_TYPE_TREE_VIEW);

enum {
	COLUMN_FILE_NAME = 0,
	COLUMN_PIXBUF
} SplashColumn;

void
splash_tree_selection_changed (GtkTreeSelection *selection,
			       GtkImage         *image)
{
	GdkPixbuf *pixbuf;
	gchar *filename, *path;

	GtkTreeModel *model;
	GtkTreeIter   iter;

	if (!gtk_tree_selection_get_selected (selection,
					      &model,
					      &iter))
		return;

	gtk_tree_model_get (model, &iter,
			    COLUMN_PIXBUF, &pixbuf,
			    -1);

	if (!pixbuf) {
		gtk_tree_model_get (model, &iter,
				    COLUMN_FILE_NAME, &filename,
				    -1);
		path = g_build_filename ("/usr/share/pixmaps/splash", filename);

		pixbuf = gdk_pixbuf_new_from_file (path, NULL);

		g_free (path);
		g_free (filename);
	}

	gtk_image_set_from_pixbuf (image, pixbuf);
}

static void
splash_tree_view_fill_from_vfs (SplashTreeView *prefs)
{
	GnomeVFSDirectoryHandle *handle;
	GnomeVFSFileInfo        *file_info;
	GnomeVFSResult           res;

	GtkListStore *store;
	GtkTreeIter   iter;

	res = gnome_vfs_directory_open (&handle, "/usr/share/pixmaps/splash",
					GNOME_VFS_FILE_INFO_GET_MIME_TYPE);

	g_warning (gnome_vfs_result_to_string (res));

	file_info = gnome_vfs_file_info_new ();
	store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (prefs)));

	while (1) {
		res = gnome_vfs_directory_read_next (handle, file_info);

		if (file_info->type == GNOME_VFS_FILE_TYPE_REGULAR) {
			gtk_list_store_append (store, &iter);
			gtk_list_store_set (store, &iter,
					    COLUMN_FILE_NAME, file_info->name,
					    COLUMN_PIXBUF, NULL,
					    -1);
		}

		gnome_vfs_file_info_clear (file_info);

		g_warning (gnome_vfs_result_to_string (res));

		if (res == GNOME_VFS_ERROR_EOF)
			break;
	}
}

static void
splash_tree_view_gconf_notify (GConfClient    *client,
			       guint           cnxn_id,
			       GConfEntry     *entry,
			       SplashTreeView *treeview)
{
	GConfValue *value;
	const gchar *splash_path;
	gchar *stripped_splash_path;

	value = gconf_entry_get_value (entry);
	splash_path = gconf_value_get_string (value);

	/* TODO: do something */
}


static void
splash_tree_view_init (SplashTreeView *prefs)
{
	GtkListStore *store;

	GtkCellRenderer   *renderer;
	GtkTreeViewColumn *col;

	store = gtk_list_store_new (2,
				    G_TYPE_STRING,
				    GDK_TYPE_PIXBUF);
	gtk_tree_view_set_model (GTK_TREE_VIEW (prefs),
				 GTK_TREE_MODEL (store));
	g_object_unref (G_OBJECT (store));

	renderer = gtk_cell_renderer_text_new ();
	col = gtk_tree_view_column_new_with_attributes ("File Name", renderer,
							"text", COLUMN_FILE_NAME,
							NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (prefs), col);

	splash_tree_view_fill_from_vfs (prefs);
}

static void
splash_tree_view_class_init (SplashTreeViewClass *class)
{
}

static void
freeee (gpointer mem)
{
	g_warning ("iiih");
}

GtkWidget *
splash_tree_view_new (void)
{
	GtkWidget   *treeview;
	GConfClient *client;

	treeview = g_object_new (TYPE_SPLASH_TREE_VIEW, NULL);

	client = gconf_client_get_default ();

	gconf_client_add_dir (client,
			      "/apps/gnome-session/options",
			      GCONF_CLIENT_PRELOAD_NONE,
			      NULL);
	gconf_client_notify_add (client,
				 "/apps/gnome-session/options/splash_image",
				 (GConfClientNotifyFunc) splash_tree_view_gconf_notify,
				 treeview,
				 (GFreeFunc) freeee,
				 NULL);

	return treeview;
}
